import React, { createContext, Component, Fragment } from 'react';
import { autobind } from 'core-decorators';
import firebase from 'firebase/app';
import 'firebase/auth';
import invariant from 'invariant';
import 'firebase/firestore';
import 'firebase/storage';

var AuthContext = createContext({});

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var _class;

var AuthProvider = autobind(_class = function (_Component) {
  inherits(AuthProvider, _Component);

  function AuthProvider(props) {
    classCallCheck(this, AuthProvider);

    var _this = possibleConstructorReturn(this, (AuthProvider.__proto__ || Object.getPrototypeOf(AuthProvider)).call(this, props));

    _this.state = {
      user: firebase.auth().currentUser,
      ready: false
    };

    _this.promise = new Promise(function (resolve) {
      _this.resolve = resolve;
    });
    return _this;
  }

  createClass(AuthProvider, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      firebase.auth().onAuthStateChanged(function (user) {
        var isReady = _this2.state.ready;
        if (!isReady || user !== _this2.state.user) {
          _this2.setState({ user: user, ready: true }, !isReady ? function () {
            return _this2.resolve();
          } : function () {});
        }
      });
    }
  }, {
    key: 'loginWithProvider',
    value: function loginWithProvider(provider) {
      return firebase.auth().signInWithPopup(provider);
    }
  }, {
    key: 'loginWithGoogle',
    value: function loginWithGoogle(options) {
      var _ref = options || {},
          scope = _ref.scope;

      var googleProvider = new firebase.auth.GoogleAuthProvider();
      googleProvider.addScope('https://www.googleapis.com/auth/' + (scope || 'contacts.readonly'));
      return this.loginWithProvider(googleProvider);
    }
  }, {
    key: 'loginWithFacebook',
    value: function loginWithFacebook(options) {
      var _ref2 = options || {},
          scope = _ref2.scope;

      var facebookProvider = new firebase.auth.FacebookAuthProvider();
      facebookProvider.addScope(scope || 'user_birthday');
      return this.loginWithProvider(facebookProvider);
    }
  }, {
    key: 'logout',
    value: function logout() {
      return firebase.auth().signOut();
    }
  }, {
    key: 'getUserData',
    value: function getUserData() {
      var _state = this.state,
          user = _state.user,
          ready = _state.ready;

      if (ready) {
        return user;
      }
      throw this.promise;
    }
  }, {
    key: 'render',
    value: function render() {
      return React.createElement(
        AuthContext.Provider,
        {
          value: {
            getUserData: this.getUserData,
            loginWithProvider: this.loginWithProvider,
            loginWithGoogle: this.loginWithGoogle,
            loginWithFacebook: this.loginWithFacebook,
            logout: this.logout
          }
        },
        this.props.children
      );
    }
  }]);
  return AuthProvider;
}(Component)) || _class;

var withAuth = function withAuth(WrappedComponent) {
  return function (_Component) {
    inherits(WithAuth, _Component);

    function WithAuth() {
      classCallCheck(this, WithAuth);
      return possibleConstructorReturn(this, (WithAuth.__proto__ || Object.getPrototypeOf(WithAuth)).apply(this, arguments));
    }

    createClass(WithAuth, [{
      key: 'render',
      value: function render() {
        var _this2 = this;

        return React.createElement(
          AuthContext.Consumer,
          null,
          function (value) {
            return React.createElement(WrappedComponent, _extends({ auth: value }, _this2.props));
          }
        );
      }
    }]);
    return WithAuth;
  }(Component);
};

var Auth = function (_Component) {
  inherits(Auth, _Component);

  function Auth() {
    classCallCheck(this, Auth);
    return possibleConstructorReturn(this, (Auth.__proto__ || Object.getPrototypeOf(Auth)).apply(this, arguments));
  }

  createClass(Auth, [{
    key: 'render',
    value: function render() {
      return React.createElement(
        AuthContext.Consumer,
        null,
        this.props.children
      );
    }
  }]);
  return Auth;
}(Component);

var FirestoreContext = createContext({});

var _class$1;

var FirestoreProvider = autobind(_class$1 = function (_Component) {
  inherits(FirestoreProvider, _Component);

  function FirestoreProvider(props) {
    classCallCheck(this, FirestoreProvider);

    // State is used for caching snapshot data
    var _this = possibleConstructorReturn(this, (FirestoreProvider.__proto__ || Object.getPrototypeOf(FirestoreProvider)).call(this, props));

    _this.state = {};

    _this.firestore = firebase.firestore();
    var query = props.query,
        children = props.children,
        customSettings = objectWithoutProperties(props, ['query', 'children']);

    _this.firestore.settings(_extends({}, customSettings, { timestampsInSnapshots: true }));
    return _this;
  }

  /**
   * Creates unique hashes from query objects. Needs to be able to handle
   * both list queries and document requests
   * @param  {Object} query
   * @return {String}
   */


  createClass(FirestoreProvider, [{
    key: 'queryHash',
    value: function queryHash(query) {
      var hash = void 0;
      if (query._query) {
        var segments = query._query.path.segments;
        var filters = query._query.filters;
        var orderBy = query._query.orderBy;

        hash = segments.join('|');
        hash += '::';
        hash += filters.map(function (f) {
          return f.field.segments.join('|') + f.op.name + f.value.toString();
        }).join('+');
        hash += '::';
        hash += orderBy.map(function (o) {
          return o.field.segments.join('|') + '|' + o.dir.name;
        }).join('+');
      } else {
        hash = query.path;
      }
      return hash;
    }

    /**
     * Basically just handles the two cases of single and multiple queries by
     * checking weather or not the response of the query function is an Array
     * @param  {Function} queryFunction
     * @return {Object|Array}
     */

  }, {
    key: 'query',
    value: function query(queryFunction) {
      var _this2 = this;

      var q = queryFunction({ firestore: this.firestore });
      return Array.isArray(q) ? q.map(function (query) {
        return _this2.queryResolve(query);
      }) : this.queryResolve(q);
    }

    /**
     * This is where the "magic" happens .. the given query gets resolved by either
     * returning the requested data from the cache if available, or throwing a promise
     * (and therefore suspending the rendering), requesting the data from firebase and
     * storing the data in the cache (while registering the snapshot listener), before
     * resolving the thrown promise (which will cause a rerender)
     * @param  {Object} query
     * @return {Object}
     */

  }, {
    key: 'queryResolve',
    value: function queryResolve(query) {
      var _this3 = this;

      // Single query
      invariant(query.onSnapshot, 'Invalid query passed! You must pass in a query that provides the ' + '`onSnapshot` method!');

      var hash = this.queryHash(query);
      if (this.state[hash]) {
        return this.state[hash];
      } else {
        throw new Promise(function (resolve, reject) {
          query.onSnapshot(function (snapshot) {
            if (snapshot.forEach) {
              // Query snapshot
              // Loop through the snapshot items and build up a list array
              var list = [];
              snapshot.forEach(function (item) {
                list.push({
                  id: item.id,
                  data: item.data(),
                  __snapshot: item,
                  __list: snapshot
                });
              });
              _this3.setState(function (state) {
                var newState = _extends({}, state);
                newState[hash] = list;
                list.forEach(function (item) {
                  // Seperately, push individual items as well
                  // This way we benefit from already loaded items from lists
                  // when we request them as items later on! :)
                  newState[_this3.queryHash(item.__snapshot.ref)] = {
                    id: item.id,
                    data: item.data,
                    __snapshot: item.__snapshot
                  };
                });
                return newState;
              }, resolve);
            } else {
              // Item snapshot
              _this3.setState(function (state) {
                return _extends({}, state, defineProperty({}, _this3.queryHash(snapshot.ref), {
                  id: snapshot.id,
                  data: snapshot.data(),
                  __snapshot: snapshot
                }));
              }, resolve);
            }
          });
        });
      }
    }
  }, {
    key: 'render',
    value: function render() {
      return React.createElement(
        FirestoreContext.Provider,
        {
          value: {
            data: this.state,
            query: this.query,
            firestore: this.firestore
          }
        },
        this.props.children
      );
    }
  }]);
  return FirestoreProvider;
}(Component)) || _class$1;

var withFirestore = function withFirestore(WrappedComponent) {
  return function (_Component) {
    inherits(WithFirestore, _Component);

    function WithFirestore() {
      classCallCheck(this, WithFirestore);
      return possibleConstructorReturn(this, (WithFirestore.__proto__ || Object.getPrototypeOf(WithFirestore)).apply(this, arguments));
    }

    createClass(WithFirestore, [{
      key: 'render',
      value: function render() {
        var _this2 = this;

        return React.createElement(
          FirestoreContext.Consumer,
          null,
          function (_ref) {
            var firestore = _ref.firestore,
                data = _ref.data,
                query = _ref.query;

            firestore.data = data;
            firestore.query = query;
            return React.createElement(WrappedComponent, _extends({ firestore: firestore }, _this2.props));
          }
        );
      }
    }]);
    return WithFirestore;
  }(Component);
};

var _class$2;

var Firestore = withFirestore(_class$2 = autobind(_class$2 = function (_Component) {
  inherits(Firestore, _Component);

  function Firestore() {
    classCallCheck(this, Firestore);
    return possibleConstructorReturn(this, (Firestore.__proto__ || Object.getPrototypeOf(Firestore)).apply(this, arguments));
  }

  createClass(Firestore, [{
    key: 'fetchData',
    value: function fetchData() {
      invariant(this.props.query, 'You need to pass a `query` to `<Firestore />`!');
      return this.props.firestore.query(this.props.query);
    }
  }, {
    key: 'render',
    value: function render() {
      var firestore = this.props.firestore;

      var data = this.fetchData();
      return React.createElement(
        Fragment,
        null,
        this.props.children({ firestore: firestore, data: data })
      );
    }
  }]);
  return Firestore;
}(Component)) || _class$2) || _class$2;

var StorageContext = createContext({});

var _class$3;

var StorageProvider = autobind(_class$3 = function (_Component) {
  inherits(StorageProvider, _Component);

  function StorageProvider() {
    classCallCheck(this, StorageProvider);

    var _this = possibleConstructorReturn(this, (StorageProvider.__proto__ || Object.getPrototypeOf(StorageProvider)).call(this));

    _this.storage = firebase.storage();
    return _this;
  }

  createClass(StorageProvider, [{
    key: 'render',
    value: function render() {
      return React.createElement(
        StorageContext.Provider,
        {
          value: {
            storage: this.storage
          }
        },
        this.props.children
      );
    }
  }]);
  return StorageProvider;
}(Component)) || _class$3;

var withStorage = function withStorage(WrappedComponent) {
  return function (_Component) {
    inherits(WithStorage, _Component);

    function WithStorage() {
      classCallCheck(this, WithStorage);
      return possibleConstructorReturn(this, (WithStorage.__proto__ || Object.getPrototypeOf(WithStorage)).apply(this, arguments));
    }

    createClass(WithStorage, [{
      key: 'render',
      value: function render() {
        var _this2 = this;

        return React.createElement(
          StorageContext.Consumer,
          null,
          function (_ref) {
            var storage = _ref.storage;

            return React.createElement(WrappedComponent, _extends({ storage: storage }, _this2.props));
          }
        );
      }
    }]);
    return WithStorage;
  }(Component);
};

var Storage = function (_Component) {
  inherits(Storage, _Component);

  function Storage() {
    classCallCheck(this, Storage);
    return possibleConstructorReturn(this, (Storage.__proto__ || Object.getPrototypeOf(Storage)).apply(this, arguments));
  }

  createClass(Storage, [{
    key: 'render',
    value: function render() {
      return React.createElement(
        StorageContext.Consumer,
        null,
        this.props.children
      );
    }
  }]);
  return Storage;
}(Component);

var _class$4;

var FirebaseContext = createContext({});

var initialised = false;

var FirebaseProvider = autobind(_class$4 = function (_Component) {
  inherits(FirebaseProvider, _Component);

  function FirebaseProvider(props) {
    classCallCheck(this, FirebaseProvider);

    var _this = possibleConstructorReturn(this, (FirebaseProvider.__proto__ || Object.getPrototypeOf(FirebaseProvider)).call(this, props));

    var config = _this.getConfig(props);
    _this.state = {
      config: config
    };

    if (!initialised) {
      firebase.initializeApp(config);
      initialised = true;
    }
    return _this;
  }

  createClass(FirebaseProvider, [{
    key: 'getConfig',
    value: function getConfig(props) {
      return {
        apiKey: props.apiKey,
        authDomain: props.authDomain,
        databaseURL: props.databaseURL,
        projectId: props.projectId,
        storageBucket: props.storageBucket,
        messagingSenderId: props.messagingSenderId
      };
    }
  }, {
    key: 'render',
    value: function render() {
      return React.createElement(
        FirebaseContext.Provider,
        { value: this.state },
        this.props.children
      );
    }
  }]);
  return FirebaseProvider;
}(Component)) || _class$4;

export { FirebaseContext, FirebaseProvider, FirestoreContext, FirestoreProvider, Firestore, withFirestore, AuthContext, AuthProvider, withAuth, Auth, StorageContext, StorageProvider, withStorage, Storage };
